/*
===============================================================================
 Name        : main.c
 Author      : 
 Version     :
 Copyright   : Copyright (C) 
 Description : main definition
===============================================================================
*/

#ifdef __USE_CMSIS
#include "LPC11xx.h"
#endif

#include <cr_section_macros.h>
#include <NXP/crp.h>

// Variable to store CRP value in. Will be placed automatically
// by the linker when "Enable Code Read Protect" selected.
// See crp.h header for more information
__CRP const unsigned int CRP_WORD = CRP_NO_CRP ;

// TODO: insert other include files here
#include <stdio.h>
#include "delay.h"
#include "diskio.h"
#include "ff.h"
#include "gpio.h"
#include "gps.h"
#include "lcd.h"
#include "minilib.h"
#include "time.h"
#include "uart.h"

extern void init_display();

// TODO: insert other definitions and declarations here

extern volatile uint32_t UARTCount;
extern volatile uint8_t UARTBuffer[BUFSIZE];
void init_uart(void)
{
	UARTInit(9600);
	return;
}

static
void uart_puts(char *str)
{
	UARTSend((uint8_t *)str, strlen(str));
	return;
}

static
void uart_gets(char buf[], int len)
{
	char c;
	int n;
	static int uart_rx_outp = 0;

	for(n = 0; n < len - 1; n++)
	{
		while(UARTCount == uart_rx_outp);

		c = UARTBuffer[uart_rx_outp];
		uart_rx_outp = (uart_rx_outp + 1) % BUFSIZE;

		buf[n] = c;

		if('\n' == c)
		{
			n++;
			break;
		}
	}
	buf[n] = '\0';

	return;
}

int main(void) {
	
	// TODO: insert code here

	static char rx_buf[128];
	char disp_buf[10*6];
	int saved_time = 0;
	int write_count;
	int count_all, count_stop;
	double speed, trip;

	uint32_t period = SystemCoreClock / 1000;	//     1msɐݒ
	SysTick_Config(period);				//     SysTick ^C}̏

	init_display();
	lcd_clear();
	lcd_puts(LCD_CENTER, 1, "Welcome!", FCOLOR, BCOLOR);
	lcd_puts(LCD_CENTER, 2, "GPS LOGGER", FCOLOR, BCOLOR);

	init_uart();
	uart_puts("test start\r\n");
//	printf("test start\r\n");

	time_clear_time();

	// Enter an infinite loop, just incrementing a counter

	while(1)
	{
		//J[h}҂
		if(0 != (disk_initialize(0) & (STA_NOINIT | STA_NODISK)))
		{
			static int n = 0;
			if(n++ % 2)
			{
				lcd_puts(LCD_CENTER, 3, "NO CARD!", RED, BCOLOR);
			}
			else
			{
				lcd_puts(LCD_CENTER, 3, "        ", BCOLOR, BCOLOR);
			}
			uart_puts("disk error\r\n");

			//1b҂
			do
			{
				uart_gets(rx_buf, sizeof(rx_buf));
			}
			while(0 != strncmp(rx_buf, "$GPRMC", 6));
			uart_puts(rx_buf);
			gps_get_time(rx_buf);

			//oߎ
			if(saved_time != time_get_time())
			{
				saved_time = time_get_time();

				int e_hour = saved_time / 3600;
				int e_min = (saved_time % 3600) / 60;
				int e_sec = (saved_time % 3600) % 60;

				itoa02(e_hour, disp_buf);
				strcat(disp_buf, ":");
				itoa02(e_min, &disp_buf[strlen(disp_buf)]);
				strcat(disp_buf, ":");
				itoa02(e_sec, &disp_buf[strlen(disp_buf)]);
				lcd_puts(LCD_RIGHT, 0, disp_buf, FCOLOR, BCOLOR);
				uart_puts(disp_buf);
				uart_puts("\r\n");
			}

			continue;
		}

		FIL fobj;
		FRESULT res;
		UINT size;
		FATFS fatfs;

		if(FR_OK != f_mount(0, &fatfs))
		{
			uart_puts("f_mount error\r\n");
			continue;
		}

		//f[^LH
		uart_gets(rx_buf, sizeof(rx_buf));
		uart_puts(rx_buf);
		if(0 == gps_get_time(rx_buf))
		{
			//󋵂ʕ\
			if(0 == strncmp(rx_buf, "$GPRMC", 6))
			{
				static int n = 0;
				if(n++ % 2)
				{
					lcd_puts(LCD_CENTER, 3, "NO DATA!", RED, BCOLOR);
				}
				else
				{
					lcd_puts(LCD_CENTER, 3, "        ", BCOLOR, BCOLOR);
				}
			}

			//oߎ
			if(saved_time != time_get_time())
			{
				saved_time = time_get_time();

				int e_hour = saved_time / 3600;
				int e_min = (saved_time % 3600) / 60;
				int e_sec = (saved_time % 3600) % 60;

				itoa02(e_hour, disp_buf);
				strcat(disp_buf, ":");
				itoa02(e_min, &disp_buf[strlen(disp_buf)]);
				strcat(disp_buf, ":");
				itoa02(e_sec, &disp_buf[strlen(disp_buf)]);
				lcd_puts(LCD_RIGHT, 0, disp_buf, FCOLOR, BCOLOR);
				uart_puts(disp_buf);
				uart_puts("\r\n");
			}

			continue;
		}

		//擾
		// "yyyymmddhhmmss.nmea" t@CƂ
		char filename[strlen("yyyymmddhhmmss.nmea") + 1];
		itoa(year, filename);
		itoa02(month, &filename[strlen(filename)]);
		itoa02(day, &filename[strlen(filename)]);
		itoa02(hour, &filename[strlen(filename)]);
		itoa02(min, &filename[strlen(filename)]);
		itoa02(sec, &filename[strlen(filename)]);
		strcat(filename, ".nmea");

		res = f_open(&fobj, filename, FA_WRITE | FA_CREATE_NEW);
		if(FR_OK != res)
		{
			uart_puts("f_open error\r\n");
			continue;
		}

		res = f_write(&fobj, (const void*)rx_buf, strlen(rx_buf), &size);
		if(FR_OK != res)
		{
			uart_puts("f_write error\r\n");
			continue;
		}

		write_count = 1;

		count_all = 1;
		count_stop = 1;
		trip = 0;

		uart_puts("log start\r\n");
		lcd_clear();

		while(1)
		{
			uart_gets(rx_buf, sizeof(rx_buf));
			uart_puts(rx_buf);

			res = f_write(&fobj, (const void*)rx_buf, strlen(rx_buf), &size);
			if(FR_OK != res)
			{
				uart_puts("f_write error\r\n");
				break;
			}

			// 100sƂɏ݃tbV
			if(100 < ++write_count)
			{
				f_sync(&fobj);
				uart_puts("sync\r\n");
				write_count = 0;
			}

			//ݎ
			if(gps_get_time(rx_buf))
			{
				itoa02(hour, disp_buf);
				strcat(disp_buf, ":");
				itoa02(min, &disp_buf[strlen(disp_buf)]);
				strcat(disp_buf, ":");
				itoa02(sec, &disp_buf[strlen(disp_buf)]);
				lcd_puts(LCD_RIGHT, 0, disp_buf, FCOLOR, BCOLOR);
			}

			//oߎ
			if(saved_time != time_get_time())
			{
				saved_time = time_get_time();

				int e_hour = saved_time / 3600;
				int e_min = (saved_time % 3600) / 60;
				int e_sec = (saved_time % 3600) % 60;

				itoa02(e_hour, disp_buf);
				strcat(disp_buf, ":");
				itoa02(e_min, &disp_buf[strlen(disp_buf)]);
				strcat(disp_buf, ":");
				itoa02(e_sec, &disp_buf[strlen(disp_buf)]);
				lcd_puts(LCD_RIGHT, 1, disp_buf, FCOLOR, BCOLOR);
			}

			speed = gps_get_speed(rx_buf);
			if(-1 != speed)
			{
				//x
				int ispeed =(int)speed;

				//3.6km/h(=1m/s)ȉ͒~ԂƂ
				if(3600 > ispeed)ispeed = 0;

				int km = ispeed / 1000;
				int m10 = (ispeed % 1000) / 10;

				itoa_3(km, disp_buf);
				strcat(disp_buf, ".");
				itoa02(m10, &disp_buf[strlen(disp_buf)]);
				strcat(disp_buf, "km/h");

				if(100 <= km)
					lcd_puts(LCD_RIGHT, 2, disp_buf, RED, BCOLOR);
				else
				if(60 <= km)
					lcd_puts(LCD_RIGHT, 2, disp_buf, YELLOW, BCOLOR);
				else
					lcd_puts(LCD_RIGHT, 2, disp_buf, FCOLOR, BCOLOR);

				//ړ
				trip += ispeed * 1000 / 3600;	//(L[g)b([g)ɕϊĉZ
				int itrip = (int)trip;

				km = itrip / 1000000;
				int m = (itrip % 1000000) / 1000;
				int cm = itrip % 1000;

				if(0 == km)
				{
					strcpy(disp_buf, "    ");
					itoa_3(m, &disp_buf[strlen(disp_buf)]);
				}
				else
				{
					itoa_3(km, disp_buf);
					strcat(disp_buf, "k");
					itoa03(m, &disp_buf[strlen(disp_buf)]);
				}
				strcat(disp_buf, "m");
				itoa03(cm, &disp_buf[strlen(disp_buf)]);
				lcd_puts(LCD_RIGHT, 3, disp_buf, FCOLOR, BCOLOR);

				//~
				count_all++;
				if(0 == ispeed)
				{
					count_stop++;
				}
				int r = count_stop * 100 * 10 / count_all;
				int rh = r / 10;
				int rl = r % 10;

				itoa_3(rh, disp_buf);
				strcat(disp_buf, ".");
				itoa(rl, &disp_buf[strlen(disp_buf)]);
				strcat(disp_buf, "%");
				lcd_puts(LCD_RIGHT, 4, disp_buf, FCOLOR, BCOLOR);
			}

			//Aeix
			if(gps_get_antenna_height(rx_buf, &disp_buf[0]))
			{
				strcat(disp_buf, "m");
				while(7 > strlen(disp_buf))
				{
					strcat(disp_buf, " ");
				}
				lcd_puts(LCD_LEFT, 4, disp_buf, FCOLOR, BCOLOR);
			}

			//q
			if(gps_get_number_of_satellites(rx_buf, disp_buf))
			{
				itoa_2(atoi(disp_buf), disp_buf);
				lcd_puts(LCD_LEFT, 2, disp_buf, FCOLOR, BCOLOR);
			}

			//is
			if(gps_get_course_over_ground(rx_buf, disp_buf))
			{
				const char dir[][4] =
				{
					" N ", "NNE", "NE ", "NEE",
					" E ", "EES", "ES ", "ESS",
					" S ", "SSW", "SW ", "SWW",
					" W ", "WWN", "WN ", "WNN",
					" N "
				};
				int d = atoi(disp_buf);
				lcd_puts(LCD_LEFT, 0, dir[(d * 10 + 225/2) / 225], FCOLOR, BCOLOR);

				itoa03(d, disp_buf);
				lcd_puts(LCD_LEFT, 1, disp_buf, FCOLOR, BCOLOR);
			}
		}
	}
	return 0 ;
}

extern void disk_timerproc (void);

// SysTicknh
void SysTick_Handler(void)
{
	disk_timerproc();
	Timer++;
	return;
}

DWORD get_fattime (void)
{
	/* Pack date and time into a DWORD variable */
	return	  ((DWORD)(year - 1980) << 25)
			| ((DWORD)month << 21)
			| ((DWORD)day << 16)
			| ((DWORD)hour << 11)
			| ((DWORD)min << 5)
			| ((DWORD)sec >> 1);
}
