#include "LPC11xx.h"
#include "integer.h"
#include "spi.h"

void init_spi(void)
{
    uint32_t i;
    uint32_t Dummy;

#if 0
    // Initialize SPI Port
    //     PIO0_2 : SSEL
    //     PIO0_6 : SCLK
    //     PIO0_9 : MOSI
    //     PIO0_8 : MISO
    GPIOSetDir(1, 4, 1); // PIO1_4 out
	LPC_IOCON->PIO1_4  = 0xd0; // SSEL, disable pu/pd mos
    GPIOSetValue(1, 4, 1); // high

    GPIOSetDir(0, 2, 1); // PIO0_2 out
	LPC_IOCON->PIO0_2  = 0xd0; // SSEL, disable pu/pd mos
    GPIOSetValue(0, 2, 1); // high
#endif

    LPC_IOCON->SCK_LOC = 0x00000002; // SCK is PIO0_6
    LPC_IOCON->PIO0_6  = 0x00000002; // SCK, disable pu/pd mos
    LPC_IOCON->PIO0_9 = 0x00000001; // MOSI, disable pu/pd mos
    LPC_IOCON->PIO0_8 = 0x00000011; // MISO, enable pu/pd mos
    //
    // Initialize SPI
    //
    LPC_SYSCON->PRESETCTRL |= (0x1<<0);
    LPC_SYSCON->SYSAHBCLKCTRL |= (1<<11);
    LPC_SYSCON->SSP0CLKDIV = 0x01; // Divided by 1

#if 0
    if (speed == SPI_FAST)
    {
        // Data Length = bitlen
        // Frame format = SPI, CPOL = 0, CPHA = 0
        // Bit Frequency = 24MHz, PCLK=48MHz
        //     CPSDVSR*(SCR+1)=2, SCR=0, CPSR=2
        LPC_SSP0->CR0 = 0x0000 + (bitlen - 1);
        LPC_SSP0->CPSR = 2;
    }
    else // if (speed == SPI_SLOW)
#endif
    {
        // Data Length = bitlen
        // Frame format = SPI, CPOL = 0, CPHA = 0
        // Bit Frequency = 0.4MHz, PCLK=48MHz
        //     CPSDVSR*(SCR+1)=120, SCR=3, CPSR=30
        LPC_SSP0->CR0 = 0x0300 + (8 - 1);
        LPC_SSP0->CPSR = 30;
    }

    // Clear the RxFIFO
    for ( i = 0; i < 8; i++ ) Dummy = LPC_SSP0->DR;

//    // Enable the SSP Interrupt
//    NVIC_EnableIRQ(SSP0_IRQn);
//    NVIC_SetPriority(SSP0_IRQn, 3);

    // Device select as master, SSP Enabled
    LPC_SSP0->CR1 = (1 << 1);	//SSPCR1_SSE

    // Enable RXMIS interrupt
  //LPC_SSP0->IMSC = SSPIMSC_RXIM;
	return;
}

void dly_100us(void)
{
	return;
}

/* SSP Status register */
#define SSPSR_TFE       (1 << 0)
#define SSPSR_TNF       (1 << 1)
#define SSPSR_RNE       (1 << 2)
#define SSPSR_RFF       (1 << 3)
#define SSPSR_BSY       (1 << 4)

/* SSP CR0 register */
#define SSPCR0_DSS      (1 << 0)
#define SSPCR0_FRF      (1 << 4)
#define SSPCR0_SPO      (1 << 6)
#define SSPCR0_SPH      (1 << 7)
#define SSPCR0_SCR      (1 << 8)

/* SSP CR1 register */
#define SSPCR1_LBM      (1 << 0)
#define SSPCR1_SSE      (1 << 1)
#define SSPCR1_MS       (1 << 2)
#define SSPCR1_SOD      (1 << 3)

/* SSP Interrupt Mask Set/Clear register */
#define SSPIMSC_RORIM   (1 << 0)
#define SSPIMSC_RTIM    (1 << 1)
#define SSPIMSC_RXIM    (1 << 2)
#define SSPIMSC_TXIM    (1 << 3)

/* SSP0 Interrupt Status register */
#define SSPRIS_RORRIS   (1 << 0)
#define SSPRIS_RTRIS    (1 << 1)
#define SSPRIS_RXRIS    (1 << 2)
#define SSPRIS_TXRIS    (1 << 3)

/* SSP0 Masked Interrupt register */
#define SSPMIS_RORMIS   (1 << 0)
#define SSPMIS_RTMIS    (1 << 1)
#define SSPMIS_RXMIS    (1 << 2)
#define SSPMIS_TXMIS    (1 << 3)

/* SSP0 Interrupt clear register */
#define SSPICR_RORIC    (1 << 0)
#define SSPICR_RTIC     (1 << 1)

void xmit_spi(BYTE d)
{
    // Tx
    while ((LPC_SSP0->SR & (SSPSR_TNF|SSPSR_BSY)) != SSPSR_TNF);

    // Send a byte
    LPC_SSP0->DR = d; //txdata;
    // Wait until the Busy bit is cleared
    while ( LPC_SSP0->SR & SSPSR_BSY );

    // Rx
    while ((LPC_SSP0->SR & SSPSR_RNE) != SSPSR_RNE);

    return;
}

BYTE rcv_spi(void)
{
    uint32_t rxdata;

    // Tx
    while ((LPC_SSP0->SR & (SSPSR_TNF|SSPSR_BSY)) != SSPSR_TNF);

    // Send a byte
    LPC_SSP0->DR = 0xff; //txdata;
    // Wait until the Busy bit is cleared
    while ( LPC_SSP0->SR & SSPSR_BSY );

    // Rx
    while ((LPC_SSP0->SR & SSPSR_RNE) != SSPSR_RNE);
    rxdata = LPC_SSP0->DR;

    return (BYTE)rxdata;
}

BYTE spi_xchg(BYTE txd)
{
    uint32_t rxdata;

    // Tx
    while ((LPC_SSP0->SR & (SSPSR_TNF|SSPSR_BSY)) != SSPSR_TNF);

    // Send a byte
    LPC_SSP0->DR = txd;
    // Wait until the Busy bit is cleared
    while ( LPC_SSP0->SR & SSPSR_BSY );

    // Rx
    while ((LPC_SSP0->SR & SSPSR_RNE) != SSPSR_RNE);
    rxdata = LPC_SSP0->DR;

    return (BYTE)rxdata;
}
