#include "delay.h"
#include "lcd.h"
#include "LPC11xx.h"

#define Delayms(a)	delay_ms(a)

#define SD_CS		(1 << 2)	//PIO0_2
#define RD			(1 << 4)	//PIO0_4
#define LCD_nRESET	(1 << 0)	//PIO1_0
#define WR			(1 << 1)	//PIO1_1
#define RS			(1 << 2)	//PIO1_2

#define RD_H			LPC_GPIO0->DATA |= RD
#define RD_L			LPC_GPIO0->DATA &= ~RD
#define WR_H			LPC_GPIO1->DATA |= WR
#define WR_L			LPC_GPIO1->DATA &= ~WR
#define RS_H			LPC_GPIO1->DATA |= RS
#define RS_L			LPC_GPIO1->DATA &= ~RS
#define LCD_nRESET_H	LPC_GPIO1->DATA |= LCD_nRESET
#define LCD_nRESET_L	LPC_GPIO1->DATA &= ~LCD_nRESET

static
void lcd_write(unsigned char index)
{
	unsigned short d;

	WR_L;

	d = LPC_GPIO0->DATA & ~((1 << 11) | (1 << 7) | (1 << 3) | (1 << 1));
	d |= (index & 0x01) ? (1 << 1) : 0;
	d |= (index & 0x02) ? (1 << 3) : 0;
	d |= (index & 0x04) ? (1 << 7) : 0;
	d |= (index & 0x08) ? (1 << 11) : 0;
	LPC_GPIO0->DATA = d;

	d = LPC_GPIO1->DATA & ~((1 << 9) | (1 << 8) | (1 << 5) | (1 << 4));
	d |= (index & 0x10) ? (1 << 4) : 0;
	d |= (index & 0x20) ? (1 << 5) : 0;
	d |= (index & 0x40) ? (1 << 8) : 0;
	d |= (index & 0x80) ? (1 << 9) : 0;
	LPC_GPIO1->DATA = d;

	WR_H;
	return;
}

static
void wr_cmd(volatile unsigned short index)
{
	RS_L;
	lcd_write((index >> 8) & 0xff);
	lcd_write((index >> 0) & 0xff);
	return;
}

static
void wr_data(volatile unsigned short data)
{
	RS_H;
	lcd_write((data >> 8) & 0xff);
	lcd_write((data >> 0) & 0xff);
	return;
}

static
void lcd_initialize(void)
{
	LCD_nRESET_L;
	Delayms(30);;
	LCD_nRESET_H;
	Delayms(30);;

	//--------------------------------ST7781 Internal Register Initial---------------------------------------//
	wr_cmd(0x00ff);wr_data(0x0001);
	wr_cmd(0x00f3);wr_data(0x0008);
	//LCD_CtrlRead_ST7781 (0x00f3); //Read Parameter
	//---------------------------End ST7781 Internal Register Initial--------------------------------------//
	//------------------------------------Display Control Setting----------------------------------------------//
	wr_cmd(0x0001);wr_data(0x0100); //Output Direct
	wr_cmd(0x0002);wr_data(0x0700); //Line Inversion
//	wr_cmd(0x0003);wr_data(0x1030); //Entry Mode (65K, BGR)
	wr_cmd(0x0003);wr_data(0x1018); //Entry Mode (65K, BGR)
	wr_cmd(0x0008);wr_data(0x0807); // Porch Setting
	wr_cmd(0x0009);wr_data(0x0000); //Scan Cycle
	wr_cmd(0x000A);wr_data(0x0000); //FMARK off
	//-----------------------------------End Display Control setting-----------------------------------------//
	//-------------------------------- Power Control Registers Initial --------------------------------------//
	wr_cmd(0x0010);wr_data(0x0790); //Power Control1
	wr_cmd(0x0011);wr_data(0x0005); //Power Control2
	wr_cmd(0x0012);wr_data(0x0000); //Power Control3
	wr_cmd(0x0013);wr_data(0x0000); //Power Control4
	//---------------------------------End Power Control Registers Initial -------------------------------//
	Delayms(100);; //Delay 100ms
	//--------------------------------- Power Supply Startup 1 Setting------------------------------------//
	wr_cmd(0x0010);wr_data(0x1490); //Power Control1
	Delayms(50); //Delay 50ms
	wr_cmd(0x0011);wr_data(0x0227); //Power Control2
	//--------------------------------- End Power Supply Startup 1 Setting------------------------------//
	Delayms(50); //Delay 50ms
	//--------------------------------- Power Supply Startup 2 Setting------------------------------------//
	wr_cmd(0x0012);wr_data(0x0089); //Power Control3
	wr_cmd(0x0013);wr_data(0x1900); //Power Control4
	wr_cmd(0x0029);wr_data(0x0021); //VCOMH setting
	//--------------------------------- End Power Supply Startup 2 Setting------------------------------//
	Delayms(50); //Delay 50ms
	//-------------------------------------Gamma Cluster Setting-------------------------------------------//
	wr_cmd(0x0030);wr_data(0x0000);
	wr_cmd(0x0031);wr_data(0x0006);
	wr_cmd(0x0032);wr_data(0x0100);
	wr_cmd(0x0035);wr_data(0x0001);
	wr_cmd(0x0036);wr_data(0x0000);
	wr_cmd(0x0037);wr_data(0x0000);
	wr_cmd(0x0038);wr_data(0x0406);
	wr_cmd(0x0039);wr_data(0x0202);
	wr_cmd(0x003c);wr_data(0x0001);
	wr_cmd(0x003d);wr_data(0x0000);
	//---------------------------------------End Gamma Setting---------------------------------------------//
	//----------------------------------Display Windows 240 X 320----------------------------------------//
	wr_cmd(0x0050);wr_data(0x0000); // Horizontal Address Start Position
	wr_cmd(0x0051);wr_data(0x00ef); // Horizontal Address End Position
	wr_cmd(0x0052);wr_data(0x0000); // Vertical Address Start Position
	wr_cmd(0x0053);wr_data(0x013f); // Vertical Address End Position
	//----------------------------------End Display Windows 240 X 320----------------------------------//
	//-------------------------------------------Frame Rate Setting-------------------------------------------//
	wr_cmd(0x0060);wr_data(0xa700); //Gate scan control
	wr_cmd(0x0061);wr_data(0x0001); //Non-display Area setting
	wr_cmd(0x0090);wr_data(0x0033); //RTNI setting
	//-------------------------------------------END Frame Rate setting------------------------------------//
	wr_cmd(0x0007);wr_data(0x0133); //Display Control1
	Delayms(50); //Delay 50ms
	
	wr_cmd(0x0022);	
}

static
unsigned short color_table[] =
{
	0x0000,	//BLACK,
	0xf800,	//RED,
	0x07e0,	//GREEN,
	0x001f,	//BLUE,
	0xffe0,	//YELLOW,
	0x07ff,	//CYAN,
	0xf81f,	//MAGENTA,
	0xffff,	//WHITE,
};

void init_display(void)
{
	LPC_IOCON->PIO0_1 = 0xd0;
	LPC_IOCON->PIO0_3 = 0xd0;
	LPC_IOCON->PIO0_4 = 0xd0;
	LPC_IOCON->PIO0_7 = 0xd0;
	LPC_IOCON->R_PIO0_11 = 0xd1;
	LPC_GPIO0->DATA |= ((1 << 11) | (1 << 7) | (1 << 4) | (1 << 3) | (1 << 1));
	LPC_GPIO0->DIR  |= ((1 << 11) | (1 << 7) | (1 << 4) | (1 << 3) | (1 << 1));

	LPC_IOCON->R_PIO1_0 = 0xd1;
	LPC_IOCON->R_PIO1_1 = 0xd1;
	LPC_IOCON->R_PIO1_2 = 0xd1;
	LPC_IOCON->PIO1_4 = 0xd0;
	LPC_IOCON->PIO1_5 = 0xd0;
	LPC_IOCON->PIO1_8 = 0xd0;
	LPC_IOCON->PIO1_9 = 0xd0;
	LPC_GPIO1->DATA |= ((1 << 9) | (1 << 8) | (1 << 5) | (1 << 4) | (1 << 2) | (1 << 1) | (1 << 0));
	LPC_GPIO1->DIR  |= ((1 << 9) | (1 << 8) | (1 << 5) | (1 << 4) | (1 << 2) | (1 << 1) | (1 << 0));

	lcd_initialize();

	unsigned int i,n,m;

#if 0
	int x = 1000;
	while(x--)
#endif
	{
		for(n=0;n<8;n++)
			for(i=0;i<LCD_SIZE_Y / 8;i++)
				for(m=0;m<LCD_SIZE_X;m++)
					wr_data(color_table[n]);

		for(i=0;i<LCD_SIZE_Y;i++)
			for(m=0;m<LCD_SIZE_X;m++)
					wr_data(color_table[7]);
	}

	for(i=0;i<LCD_SIZE_Y;i++)
		for(m=0;m<LCD_SIZE_X;m++)
				wr_data(color_table[BCOLOR]);

	return;
}

void display_area_set(int xs, int ys, int xe, int ye)
{
	wr_cmd(0x0050);wr_data(xs);	// Horizontal GRAM Start Address 
	wr_cmd(0x0051);wr_data(xe);	// Horizontal GRAM End Address 
	wr_cmd(0x0052);wr_data(ys);	// Vertical GRAM Start Address 
	wr_cmd(0x0053);wr_data(ye);	// Vertical GRAM End Address 

	wr_cmd(0x0020);wr_data(xs);	// GRAM horizontal Address 
	wr_cmd(0x0021);wr_data(ys);	// GRAM Vertical Address  

	wr_cmd(0x0022);
	return;
}

void pset(int color)
{
	wr_data(color_table[color]);
	return;
}
